//
//  SongController.m
//
//  Controller class for the dummy movie window.
//  Created by Alexander Thomas on Sun Jul 13 2003.
//  dr.lex at cryogen.com
//

#import "SongController.h"
#include <QuickTime/QuickTime.h>
#include <QuickTime/Movies.h>
#include <unistd.h>


// This thing will be used to call routines from outside the SongController object.
static void * gControlRef = NULL;


@implementation SongController

// This function is called right before the window appears
- (void)awakeFromNib
{
  /* we must initialize the movie toolbox before calling any of its functions */
  EnterMovies();
  
  saveObjectForCallback(self);
  theVolume = 1;
}


- (void) startMovie:(NSString*)fileName loop:(BOOL)loopFlag
{
  if([dummyView isPlaying])
    [dummyView stop:NULL];
  NSMovie *theMovie = [[NSMovie alloc] initWithURL:[NSURL fileURLWithPath:fileName] byReference:YES];
  [dummyView setMovie:theMovie];
  if(loopFlag)
    [dummyView setLoopMode:NSQTMovieLoopingPlayback];
  else
    [dummyView setLoopMode:NSQTMovieNormalPlayback];

  [dummyView setVolume:theVolume]; // NSMovieview resets volume when loading a movie
  [dummyView start:NULL];
}

- (void) setSongVolume:(float)volume
{
  [dummyView setVolume:volume];
  theVolume = volume;
}

- (float) getSongVolume
{
  return [dummyView volume];
}

- (BOOL) isPlaying
{
  return [dummyView isPlaying];
}

- (void) play
{
  [dummyView start:NULL];
}

- (void) stop
{
  [dummyView stop:NULL];
}

- (void*) getMovie
{
  return [[dummyView movie] QTMovie];
}

@end


// This routine stores a reference to our SongController object. We'll
// need this so we can call into methods in this class from outside the
// implementation of the class methods.
void saveObjectForCallback(void *theObject)
{
  gControlRef = theObject;
}

// Glue routines between the SongController class and the raw C functions

int QT_PlaySong(char *fileName, int loopFlag)
{
  NSString* fName = [NSString stringWithCString:fileName];
  BOOL loopIt = (loopFlag == 1);
  [(SongController*)gControlRef startMovie:fName loop:loopIt];
  return 1;
}

void QT_SetVolume(int volume)
{
  float vol = (float)volume/255.0;
  [(SongController*)gControlRef setSongVolume:vol];
}

int QT_GetVolume()
{
  float vol = [(SongController*)gControlRef getSongVolume];
  return (int)(vol*255.0F);
}

int QT_SongPlaying()
{
  return ([(SongController*)gControlRef isPlaying] == YES);
}

void QT_Continue()
{
  [(SongController*)gControlRef play];
}

void QT_Pause()
{
  [(SongController*)gControlRef stop];
}

void QT_SetSongTime(unsigned long mSec)
{
  int isPlaying;
  TimeRecord newTime;
  Movie theMovie = [(SongController*)gControlRef getMovie];

  newTime.scale = GetMovieTimeScale(theMovie);
  (newTime.value).lo = mSec*newTime.scale/1000; // see MacTypes.h and Math64.h
  (newTime.value).hi = 0; // I forgot this the first time. I almost went totally mad debugging.
  newTime.base = GetMovieTimeBase(theMovie);
  isPlaying = QT_SongPlaying();
  QT_Pause();
  SetMovieTime(theMovie, &newTime);
  if(isPlaying)
    QT_Continue();
}

unsigned long QT_GetSongTime()
{
  TimeRecord theTime;
  unsigned long result;
  Movie theMovie = [(SongController*)gControlRef getMovie];

  GetMovieTime(theMovie, &theTime);
  result = (theTime.value).lo*1000/GetMovieTimeScale(theMovie);
  return result;
}

unsigned long QT_GetSongLength()
{
  Movie theMovie = [(SongController*)gControlRef getMovie];
  TimeValue theDuration = GetMovieDuration(theMovie)*1000/GetMovieTimeScale(theMovie);
  return theDuration;
}

int QT_FadeVolume(int toVolume, int milliSeconds)
{
  int i, startVolume = QT_GetVolume();
  float increment = (float)(toVolume-startVolume)/(float)(milliSeconds);
  for(i = 0; i < milliSeconds-50; i += 50) {
    QT_SetVolume( startVolume+(int)(increment*i) );
    usleep(50000);
  }
  QT_SetVolume(toVolume);
  return 1;
}

int QT_FadeActive()
{
  // is always false because fading is done in a 'busy' way.
  return 0;
}
